local M = {} -- Author: _N_S_

local debug = false

local function getValue(jbeamValue)
	if type(jbeamValue) == "number" then
		return jbeamValue
	elseif type(jbeamValue) == "string" then
		jbeamValue = jbeamValue:gsub("%s", "")
		if tonumber(jbeamValue) then
			return tonumber(jbeamValue)
		elseif jbeamValue:sub(0,5) == "rand(" and jbeamValue:sub(-1) == ")" then
			jbeamValue = jbeamValue:sub(6,-2):split("[^,]+")
			if #jbeamValue == 2 then
				local value1 = tonumber(jbeamValue[1])
				local value2 = tonumber(jbeamValue[2])
				if value1 and value2 then
					return math.random() * (math.max(value1,value2) - math.min(value1,value2)) + math.min(value1,value2)
				end
			end
		end
	end
	
	return false
end

local function updateGFX(dt)
	local currentGearIndex = electrics.values.gearIndex

	if M.previousGearIndex ~= currentGearIndex then
		if M.previousGearIndex < currentGearIndex then
			M.currentPressure = M.currentPressure - M.costsOfSwitchingUp[currentGearIndex] -- UP
			if M.currentPressure < 0 then M.currentPressure = 0 end
		else
			M.currentPressure = M.currentPressure - M.costsOfSwitchingDown[currentGearIndex] -- DOWN
			if M.currentPressure < 0 then M.currentPressure = 0 end
		end
		M.previousGearIndex = currentGearIndex
	end

	if M.currentPressure < M.triggerPressure and electrics.values.running then
		if not M.compressorIsRunning then
			M.compressorIsRunning = true
			M.compressorLoop = M.compressorLoop or obj:createSFXSource2(M.soundFilePath, "AudioDefaultLoop3D", "compressorLoop", M.soundNode, 0)
			obj:setVolume(M.compressorLoop, M.soundVolume)
			obj:cutSFX(M.compressorLoop)
			if (playerInfo or {}).seatedPlayers then
				obj:playSFX(M.compressorLoop)
			end
		end
	end

	if M.compressorIsRunning then
		if M.currentPressure < M.targetPressure and electrics.values.running then
			M.currentPressure = M.currentPressure + M.pumpingSpeed * dt
		else
			M.compressorIsRunning = false
			obj:setVolume(M.compressorLoop, 0)
			obj:stopSFX(M.compressorLoop)
		end
	end

	if debug then dump(M.currentPressure) end
end

local function reset()
	M.currentPressure = 									   M.defaultPressure
	M.compressorIsRunning = 								   false
end

local function init(jbeamData)
	if debug then dump(jbeamData) end
	M.maxPressure = 	getValue(jbeamData.maxPressure) 	or 145
	M.defaultPressure = getValue(jbeamData.defaultPressure) or 72
	M.targetPressure =  getValue(jbeamData.targetPressure) 	or M.defaultPressure
	M.triggerPressure = getValue(jbeamData.triggerPressure) or M.targetPressure / 3
	M.pumpingSpeed = 	getValue(jbeamData.pumpingSpeed) 	or (M.targetPressure - M.triggerPressure) / 2
	M.currentPressure = 									   M.defaultPressure

	M.costsOfSwitchingUp = 									   {}
	M.costsOfSwitchingDown = 								   {}
	M.previousGearIndex = 									   0
	M.soundVolume = 	getValue(jbeamData.soundVolume) 	or 1
	M.compressorIsRunning = 								   false
	M.compressorLoop = 										   nil
	M.soundFilePath = type(jbeamData.soundFilePath) == "string" and FS:fileExists(jbeamData.soundFilePath) and jbeamData.soundFilePath or "event:>Vehicle>Cooling Fan>Mechanical_04"

	if type(jbeamData.soundNode) ~= "string" and type(jbeamData.soundNode) ~= "table" then return end

	for i=1, type(jbeamData.soundNode) == "table" and #jbeamData.soundNode or 1 do
		for _,node in pairs(v.data.nodes) do
			if node.name == type(jbeamData.soundNode) == "table" and jbeamData.soundNode[i] or jbeamData.soundNode then
				M.soundNode = node.cid
				goto BREAK
			end
		end
	end do return end

	::BREAK::

	local device = powertrain.getDevice(jbeamData.deviceName) or (powertrain.getDevicesByCategory("gearbox") or {})[1]
	if not device then return end

	if type(jbeamData.costsOfSwitchingUp) ~= "table" and type(jbeamData.costsOfSwitchingDown) ~= "table" then return end

	if type(jbeamData.costsOfSwitchingUp) == "table" then
		if type(jbeamData.costsOfSwitchingUp[1]) ~= "table" then return end
		if jbeamData.costsOfSwitchingUp[1][1] ~= "gearIndex" or jbeamData.costsOfSwitchingUp[1][2] ~= "psiCost" then return end
		if #jbeamData.costsOfSwitchingUp-1 ~= device.gearCount then return end

		for i=device.gearCount, 1, -1 do
			if getValue(jbeamData.costsOfSwitchingUp[i+1][1]) ~= device.maxGearIndex - (i-1) then return end
			M.costsOfSwitchingUp[device.maxGearIndex - (i-1)] = getValue(jbeamData.costsOfSwitchingUp[i+1][2])
		end
	end

	if type(jbeamData.costsOfSwitchingDown) == "table" then
		if type(jbeamData.costsOfSwitchingDown[1]) ~= "table" then return end
		if jbeamData.costsOfSwitchingDown[1][1] ~= "gearIndex" or jbeamData.costsOfSwitchingDown[1][2] ~= "psiCost" then return end
		if #jbeamData.costsOfSwitchingDown-1 ~= device.gearCount then return end

		for i=device.gearCount, 1, -1 do
			if getValue(jbeamData.costsOfSwitchingDown[i+1][1]) ~= device.maxGearIndex - i then return end
			M.costsOfSwitchingDown[device.maxGearIndex - i] = getValue(jbeamData.costsOfSwitchingDown[i+1][2])
		end
	end

	if type(jbeamData.costsOfSwitchingUp) ~= "table" then
		for k,v in pairs(M.costsOfSwitchingDown) do
			M.costsOfSwitchingUp[k+1] = v*0.5
		end
	elseif type(jbeamData.costsOfSwitchingDown) ~= "table" then
		for k,v in pairs(M.costsOfSwitchingUp) do
			M.costsOfSwitchingDown[k-1] = v*2
		end
	end

	M.updateGFX = updateGFX
	if debug then dump(M) end
end

M.init = init
M.reset = reset
return M